<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(FCPATH . '/application/libraries/base_table.php');

class Entity_accounts_table extends Base_table {

	function __construct()
	{
		parent::__construct(array('tbl'=>'entity_accounts', 'tbl_key'=>'entity_id'));
	}
	
	public function formatdata(&$data) 
	{
		// encrypt password
        if (!empty($data['password'])) {            
            $data['password'] = $this->CI->auth->encode($data['password']);
        }
        
        // add domain suffix
        $data['domain_url']=!empty($data['domain_url'])?($data['domain_url'].'.'.$this->CI->config->item('domain')):NULL;
                
        $data['register_date']	= $this->cur_time;
		$days = $this->CI->config->item('SMX_trial_lic_days');
		if (defined('APP_EXP')) {
			$days = $this->CI->config->item('SMX_trial_lic_saas');
		}
        $data['exp_date'] 		= date('Y-m-d',strtotime($data['register_date'] . " +".$days." days"));

        return true;
    }
    
	public function save($data)
	{
		$values = $data;
		$values['is_active'] = 1;
		if ($entity_id = parent::save($values)) {
		
			// load required files
			$this->CI->load->library('entity_users_table');
			$this->CI->load->library('entity_table');
			
			$this->CI->load->library('entity_setting_table');
			$this->CI->load->library('entity_daysoff_table');
			$this->CI->load->library('expense_type_table');
			$this->CI->load->library('task_type_table');
			$this->CI->load->library('task_status_table');
			$this->CI->load->library('user_dept_table');

			// prepare data for entity users
			$values['password']		= $data['password'];
			$values['entity_id']	= $entity_id;
			$values['domain_name']	= $data['domain_url'];
			if($this->CI->config->item('SMX_signup_with_autologin'))
			{
				$values['activation']	= $data['activation'];
				$values['auth_token']	= $data['auth_token'];
			}
			$values['user_role']	= $this->CI->config->item('SMX_entity_user_role');
			$values['department']	= $this->CI->config->item('SMX_entity_user_department');
			$values['user_image']	= isset($values['user_image'])?$values['user_image']:'';

			if ($user_id = $this->CI->entity_users_table->save($values))
			{
				// store entity details
				$values['user_id'] = $user_id;
				if (! $this->CI->entity_table->save($values))
				{
					return false;
				}

				// store default data for new entity
				if (! $this->CI->entity_setting_table->store_default_data($values))
				{
					return false;
				}

				if (! $this->CI->entity_daysoff_table->store_default_data($values))
				{
					return false;
				}

				if (! $this->CI->expense_type_table->store_default_data($values))
				{
					return false;
				}

				if (! $this->CI->task_type_table->store_default_data($values))
				{
					return false;
				}

				if (! $this->CI->task_status_table->store_default_data($values))
				{
					return false;
				}

				if (! $this->CI->user_dept_table->store_default_data($values))
				{
					return false;
				}

				return $values;
			}
		}
		return false;
	}

	public function register($data)
	{
		$values = $data;
		$values['is_active'] = 1;

		if ($entity_id = parent::save($values)) {
//			$exp_date = $values['exp_date'];

			// update license key
			$license_key = $this->CI->auth->get_license(array(
				'company_name'	=> $data['company_name'],
				'entity_id'		=> $entity_id,
				'expiry_date'	=> $values['exp_date'],
				'max_users'		=> '0',
				'plan'			=> 'T'
			));

			$this->CI->db->where('entity_id', $entity_id);
			$this->CI->db->update('entity_accounts', array('license_key' => $license_key));

			// load required files
			$this->CI->load->library('entity_setting_table');
			$this->CI->load->library('entity_users_table');
			$this->CI->load->library('entity_table');
			
			$values['password']		= $data['password'];
			$values['entity_id']	= $entity_id;
			$values['domain_name']	= $data['company_name'];
			
			$values['user_role']	= $this->CI->config->item('SMX_entity_user_role');
			$values['department']	= $this->CI->config->item('SMX_entity_user_department');
			$values['user_image']	= isset($values['user_image'])?$values['user_image']:'';
			$values['desktop_signup'] = true;

			if ($user_id = $this->CI->entity_users_table->save($values))
			{
				// store entity details
				$values['user_id'] 		= $user_id;
				// store default data for new entity
				if (! $this->CI->entity_setting_table->store_default_data($values))
				{
					return false;
				}

				$values['license_key'] 	= $license_key;

				if ($this->CI->entity_table->save($values))
				{
						return array('company_name' => $data['company_name'], 'email' => $data['email'] , 'license_key' => $values['license_key'],'user_name' => $data['user_name']);
				}
			}
		}
		return false;
	}
}

/* end of Entity_accounts_table.php */
