<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Invoices_pmf extends CI_Model
{
	private $_entity_id	= 0;

	function __construct()
	{
		parent::__construct();
		$this->load->library(OTBASE_PATH.'OT_App');
        $this->load->library(OTBASE_PATH.'OT_User');
        $this->_entity_id	= $this->ot_app->get('entity_id');
	}
	    
	public function get_invoice_defaults($with_last=false)
    {
    	$next_num = '0001';
    	
    	$this->db->select('invoice_number AS last, due_date_mode');
    	$this->db->from('invoice');
    	$this->db->where(array('entity_id' => $this->_entity_id));
    	$this->db->order_by('invoice_id', 'desc');
    	$this->db->limit(1);
    	$query	= $this->db->get();

    	$return	= $query->row_array();

    	if ($return)
    	{
    		$pattern	= '/[0-9]+$/';    		
    		if (preg_match($pattern, $return['last'],$matches))
    		{
    			$next_num	= str_pad($matches[0]+1, strlen($matches[0]), 0, STR_PAD_LEFT);
				$next_num	= preg_replace($pattern, $next_num, $return['last']);
    		}
    		else
    		{
    			$next_num	= $return['last'].'1';
    		}
    	}

    	$return['next']	= $next_num;
    	if ($with_last)
    	{
    		return $return;
    	}

    	return $next_num;
    }
	
	public function invoice_list_query($select, $options = array()) 
	{		
		$select	= ($select)?$select:"inv.invoice_id, inv.invoice_number, inv.estimate_id, DATE_FORMAT(inv.invoice_date, '%Y-%m-%d') as invoice_date, DATE_FORMAT(inv.due_date, '%Y-%m-%d') as due_date, inv.status, inv.amount, inv.payment_amount, DATE_FORMAT(inv.last_updated_date, '%Y-%m-%d') as last_updated_date, inv.recurring_mode, DATE_FORMAT(inv.recurring_next_inv_date, '%Y-%m-%d') as recurring_next_inv_date, inv.currency_code, c.client_id, concat(c.client_code, c.client_name) AS client_name, inv.is_archive";
		$this->db->select($select, false);
		$this->db->from('invoice AS inv');
		$this->db->join('client AS c', 'inv.entity_id=c.entity_id and inv.client_id = c.client_id', 'INNER');
		
		$where = array();
		if (!empty($options['filters']))
		{
			foreach($options['filters'] as $key=>$val) {
				if (is_array($val))
					$this->db->where_in($key,$val);
				else
					$where[$key] = $val;
			}
		}

		$where['inv.entity_id'] = $this->_entity_id;
		
		if (!empty($options['status']))
		{
			$where['inv.is_archive'] =  0;
			switch ($options['status'])
			{
				case 'P':
					$where['inv.status'] =  'p';
					break;
				case 'R':
					//$where['inv.recurring_mode !='] =  '0';		
					$this->db->where_not_in('inv.recurring_mode', array('','0'));
					break;
				case 'A':
					$where['inv.is_archive'] =  1;
					break;
				case 'D':
					$where['inv.status'] =  'd';
					break;
				default:
					$this->db->where_in('inv.status', array('s','b'));		
			}
		}
		
	/*	if (!empty($options['filters']))
		{
			$where = array_merge($where, $options['filters']);
		} */
		$this->db->where($where);
		
		if (!empty($options['group_by']))
		{
			$this->db->group_by($options['group_by']);
		}

		if (!empty($options['order_by']))
		{
			$order_by = $options['order_by'];
			$this->db->order_by($order_by, 'DESC');
		}

		$from = !empty($options['from'])?$options['from']:0;
		if (!empty($options['count']))
		{
			$this->db->limit($options['count'], $from);
		}

		return $this->db->get();
	}

	public function get_list($select = '', $options = array())
	{
		$options['filters'] = (!empty($options['filters']))?$options['filters']:array();
		$options['filters'] = array_merge($options['filters'], array('inv.created_by' => $this->ot_user->get('user_id'), 'inv.status' =>'d'));
		
		$query	= $this->invoice_list_query($select, $options);
		return $query->result_array();
	}
	
	public function get_status_count($select = '', $options = array())
	{
		if (!$select) $select = 'count(inv.invoice_id) AS count, case when inv.is_archive=1 then "a" else status end as status_res';
		
		return $this->get_list($select, $options);
	}

	public function get_invoice_count($client_id=0, $status='')
	{
		if (empty($client_id))
		{
			return 0;
		}

		$this->db->where('entity_id', $this->_entity_id);
		$this->db->where('client_id', $client_id);
		if (!empty($status))
		{
			if (is_array($status))
			{
				$this->db->where_in('status', $status);
			}
			else
			{
				$this->db->where('status', $status);
			}
		}

		$count	= $this->db->count_all_results('invoice');

		return $count;
	}

	public function get_invoice_details($invoice_id=null, $select=null)
	{
		if ($invoice_id)
		{
			$select	= $select ? $select : 'invi.*';
			$this->db->select($select);
			$this->db->from('invoice_items AS invi');
			if (is_array($invoice_id))
			{
				$this->db->where_in('invi.invoice_id', $invoice_id);
			}
			else
			{
				$this->db->where('invi.invoice_id', $invoice_id);
			}
			$this->db->order_by('invi.serial_no');
			$query	= $this->db->get();

			return $query->result_array();
		}

		return false;
	}
	
	public function save($data)
	{
		$this->load->library('invoice_table');
		
		$params = array();
		if (!empty($data[$this->invoice_table->tbl_key]))
		{
			$params['where'] = array('entity_id' => $this->_entity_id, 'invoice_id' => $data[$this->invoice_table->tbl_key], 'created_by' => $this->ot_user->get('user_id'), 'status' => 'd');
		}
		$result = $this->invoice_table->save($data,$params);
		 
		return $result;
	}
	
	
	public function update_invoiceids ($data, $invoice_id)
	{
		if ($data['invoice_hours']) {
			$where = array('entity_id' => $this->ot_app->get('entity_id'), 'is_billable' => 1, 'approval_status' => 'A', 'is_progress !=' => 1, 'invoice_id' => 0);
			if (!empty($data['start_date'])) {
				$where['entry_date >='] = $data['start_date']; 
			}
			if (!empty($data['finish_date'])) {
				$where['entry_date <='] = $data['finish_date']; 
			}
			$this->db->where($where);
			$this->db->where_in('project_id', $data['project_ids']);
			$this->db->update('time_sheet', array('invoice_id' => $invoice_id, 'last_updated_date' => $this->auth->get_timezone_date('','UTC')));
			
			$this->reset_unin_hours($data['project_ids']);
		}
		
		if ($data['invoice_expenses']) {
			$where = array('entity_id' => $this->_entity_id, 'approved_by >' => 0);
			if (!empty($data['start_date'])) {
				$where['expense_date >='] = $data['start_date']; 
			}
			if (!empty($data['finish_date'])) {
				$where['expense_date <='] = $data['finish_date']; 
			}
			$this->db->where($where);
			$this->db->where_in('project_id', $data['project_ids']);
			$this->db->update('project_expense', array('invoice_id' => $invoice_id));
		}
		
		return $this->db->affected_rows();
	}
	
	public function reset_unin_hours($project_ids = array()) {
		// project uninvoiced hours update
		$this->db->select('sum( case when invoice_id=0 then hours else 0 end ) as unin_hrs, project_id', false);
		$this->db->from('time_sheet');
		if ($project_ids)
			$this->db->where_in('project_id', $project_ids);
		$this->db->where(array('entity_id' => $this->ot_app->get('entity_id')));
		$this->db->group_by('project_id');
		$projects = $this->db->get()->result_array();
		if ($projects) {
			foreach ($projects as $row) {
				$this->db->where(array('entity_id' => $this->ot_app->get('entity_id'), 'project_id' => $row['project_id']));
				$this->db->update('project', array('uninvoiced_hours' => $row['unin_hrs']));
			}
		}
		return true;
	}
	public function get_import_items($data) 
	{
		$result = array();
		$project_ids = array();

		if (!is_admin()) {
			$this->load->model('projects');
			$tmp	= $this->projects->get_projects('pr.project_id,pr.project_name');
			foreach ($tmp as $key=>$val)
			{
				$project_ids[]=$val['project_id'];
			}
		}
		
		if (empty($data['task']) || $data['task']=='timesheet') { 
			$this->db->select('ts.project_id, concat(pr.project_code, pr.project_name) AS project_name, DATE_FORMAT(ts.entry_date, "%Y-%m-%d") as date', false);
			
			if (!isset($data['billable']) || $data['billable']) {
				$this->db->select("SUM(ts.hours * ts.is_billable * (case ts.approval_status when 'A' then 1 else 0 end)) AS billable_hours", FALSE); // approved
				$this->db->select("SUM(ts.hours * ts.is_billable * (case ts.approval_status when 'A' then 0 else 1 end)) AS unap_billable_hours", FALSE); // unapproved
			}
			
			if (!isset($data['billable']) || !$data['billable']) {
				$this->db->select("SUM(ts.hours*!ts.is_billable * (case ts.approval_status when 'A' then 1 else 0 end)) AS non_billable_hours", FALSE); // approved
				$this->db->select("SUM(ts.hours * !ts.is_billable * (case ts.approval_status when 'A' then 0 else 1 end)) AS unap_non_billable_hours", FALSE); // unapproved
			}
			
			$this->db->from('time_sheet as ts');
			$this->db->join('project as pr', 'ts.entity_id=pr.entity_id and pr.project_id=ts.project_id');
			$this->db->join('task_type as ty', 'ty.entity_id=ts.entity_id and ty.task_type_id=ts.task_type_id');
			$this->db->join('project_task_types as pty', 'pty.entity_id=pr.entity_id and pty.task_type_id=ty.task_type_id AND pty.project_id = pr.project_id');
			$this->db->join('project_users as pru', 'pru.entity_id=pr.entity_id and ts.user_id=pru.user_id AND pru.project_id = pr.project_id');
			$where = array('pr.client_id' => $data['client_id'], 'ts.is_progress != ' => 1,'ts.entity_id' => $this->_entity_id, 'pr.invoice_type !=' => 'N');
			if (!empty($data['start_date']) && !empty($data['finish_date']))
			{
				$where['ts.entry_date >='] = $data['start_date']; $where['ts.entry_date <='] = $data['finish_date'];
			}
			if (!empty($data['project_id']))
			{
				if (!is_array($data['project_id']))
					$this->db->where('pr.project_id', $data['project_id']);
				else if (implode('',$data['project_id'])) 
					$this->db->where_in('pr.project_id', $data['project_id']);
			} else if ($project_ids) {
					$this->db->where_in('pr.project_id', $project_ids);
			}
			$where['pr.is_active'] = 1;
			$this->db->where($where);

			if (!empty($data['invoice_id'])) {	
				$this->db->where('(ts.invoice_id<=0 OR ts.invoice_id ='.$data['invoice_id'].')');
			} else {
				$this->db->where(array('ts.invoice_id <= '=>0));
			}

			$this->db->group_by('ts.project_id');
			
			$query	= $this->db->get();
			$result = $query->result_array();
		}	

		if (empty($data['task']) || $data['task']=='expenses') {
			// get uninvoised expenses
			$this->db->select('pr.project_id, concat(pr.project_code, pr.project_name) AS project_name,DATE_FORMAT(pre.expense_date, "%Y-%m-%d") as date,  sum(pre.amount * (pre.approved_by/pre.approved_by)) AS expense_total, sum(pre.amount * (!pre.approved_by )) AS unap_expense_total', false);		
			$this->db->from('project_expense AS pre');
			$this->db->join('project as pr', 'pre.entity_id=pr.entity_id  and pr.project_id=pre.project_id');
			$where = array('pr.client_id' => $data['client_id'], 'pre.entity_id' => $this->_entity_id);
			if (!empty($data['start_date']) && !empty($data['finish_date']))
			{
				$where['pre.expense_date >='] = $data['start_date']; $where['pre.expense_date <='] = $data['finish_date'];
			}
			if (!empty($data['filter_project_id']))
			{
				if (!is_array($data['filter_project_id']))
					$this->db->where('pr.project_id', $data['filter_project_id']);
				else if ($data['filter_project_id'])
					$this->db->where_in('pr.project_id', implode('',$data['filter_project_id']));
				//$where['pr.project_id'] = $data['filter_project_id'];
			} else if ($project_ids){
				$this->db->where_in('pr.project_id', $project_ids);
			}
			$this->db->where($where);

			if (!empty($data['invoice_id'])) {	
				$this->db->where('(pre.invoice_id<=0 OR pre.invoice_id ='.$data['invoice_id'].')');
			} else {
				$this->db->where(array('pre.invoice_id <= '=>0));
			}

			$this->db->group_by('project_id');
			$query	= $this->db->get();
			$expenses = $query->result_array();
	
			if ($expenses)
			{
				$hours		= array();
				$dummy_keys = array_keys($expenses[0]);
				$dummy_expeses = array_combine($dummy_keys, array_fill(0, count($dummy_keys), null));
				$expenses	= array_key_index($expenses,'project_id');

				if ($result) {
					foreach ($result as $key=>$val) {
						$hours[$val['project_id']] = array_merge($dummy_expeses, $val);
					}
				}


				if ($hours) {
					foreach ($expenses as $key=>$row) {
						if (isset($hours[$key]))
						{
							$hours[$key] = array_merge($hours[$key], $row);
						} else {
							$hours[$key] = $row;
						}
					}
				} else {
					$hours = $expenses;
				}
				$result = array_values($hours);
			}
		}

		return $result;		
	}
	
	public function get_user_hours_details ($data) {
		$this->db->select('sum(ts.hours) as hours, ts.task_type_id, ts.user_id');
		$this->db->from('time_sheet AS ts');
		$this->db->join('project as pr', 'ts.entity_id=pr.entity_id and pr.project_id=ts.project_id');
		$where = array('ts.entity_id' => $this->_entity_id, 'ts.project_id ' => $data['project_id'], 'pr.is_active' => 1, 'ts.is_billable' => $data['billable'], 'ts.is_progress !='=> 1, 'ts.invoice_id'=> 0);	
		if (!empty($data['from_date']) && !empty($data['to_date']))
		{
			$where['ts.entry_date >='] = $data['from_date']; $where['ts.entry_date <='] = $data['to_date'];
		}	
		$this->db->where($where);
		
		$group_by = !empty($data['group_by'])?$data['group_by']:'ts.user_id';
		$this->db->group_by($group_by);
		$query	= $this->db->get();
		$result = array();
		if($tmp = $query->result_array()) {
			$result['items'] = $tmp;
			
			// get related users
			$this->db->select('eu.user_id, eu.first_name, eu.last_name, eu.user_image', false);
			$this->db->from('entity_users AS eu');
			$this->db->join('project_users AS pmu', 'pmu.entity_id = eu.entity_id and pmu.user_id=eu.user_id');
			$this->db->where(array('eu.entity_id' => $this->_entity_id, 'eu.is_active' => '1', 'pmu.project_id' => $data['project_id']));
			$query	= $this->db->get();
			$result['users'] = $query->result_array();
		}
		
		return $result;
	}

	public function update_billable($data) {
		if (empty($data['users'])) return true;
		
		foreach ($data['users'] as $user_id=>$tt_ids) {
			if ($tt_ids) {
				$where = array('entity_id' => $this->_entity_id, 'project_id' => $data['project_id'],'user_id'=>$user_id,'invoice_id' => 0 , 'approval_status' => 'A', 'is_billable !=' => $data['billable']);
				if (!empty($data['from_date']) && !empty($data['to_date']))
				{
					$where['entry_date >='] = $data['from_date']; $where['entry_date <='] = $data['to_date'];
				}
				$this->db->where($where);
				$this->db->where_in('task_type_id', $tt_ids);
				$this->db->update('time_sheet', array('is_billable' => $data['billable']));				
			}
		}

		return true;
	}
	
	public function get_uninvoiced_details ($client_id, $invoice_id=0)
	{
		$project_ids = array();

		if (!is_admin()) {
			$this->load->model('projects');
			$tmp	= $this->projects->get_projects('pr.project_id,pr.project_name');
			foreach ($tmp as $key=>$val)
			{
				$project_ids[]=$val['project_id'];
			}
		}

		$this->db->select('sum(ts.hours * ts.is_billable) AS uninvoiced_hours', false);		
		$this->db->from('time_sheet AS ts');
		$this->db->join('project as pr', 'ts.entity_id=pr.entity_id and pr.project_id=ts.project_id');
		$this->db->where(array('pr.client_id ' => $client_id, 'ts.entity_id' => $this->_entity_id));
		$this->db->where(array('ts.is_progress !='=> 1, 'ts.invoice_id'=> 0));
		if ($project_ids) {
			$this->db->where_in('pr.project_id', $project_ids);
		}
		if ($invoice_id) {
			$this->db->or_where(array('ts.invoice_id'=> $invoice_id));
		}
		
		$query	= $this->db->get();

		$result = $query->row_array();
		
		$this->db->select('sum(pe.amount) AS uninvoiced_expenses', false);		
		$this->db->from('project_expense AS pe');
		$this->db->join('project as pr', 'pe.entity_id=pr.entity_id and pr.project_id=pe.project_id');
		$this->db->where(array('pr.client_id ' => $client_id, 'pe.entity_id' => $this->_entity_id, 'pe.approved_by >' => 0));
		$this->db->where(array('pe.invoice_id'=> 0));
		if ($project_ids) {
			$this->db->where_in('pr.project_id', $project_ids);
		}
		if ($invoice_id) {
			$this->db->or_where(array('pe.invoice_id' => $invoice_id));
		}
		$query	= $this->db->get();
		$expense = $query->row_array();
		
		$result = array_merge($result, $expense);
		return $result;
	}

	public function get_import_details ($data)
	{
		$result = array();
		
		if (empty($data['project_ids']))
		{
			return $result;
		}

		$entity_id = $this->ot_app->get('entity_id')?$this->ot_app->get('entity_id'):(!empty($data['entity_id'])?$data['entity_id']:null);
		if (!$entity_id) return $result;
		
		// invoice detail by
		if (!empty($data['invoice_hours'])) {
			
			// split porjects based on invoice type
			$this->db->select('project_id,invoice_type');
			$this->db->from('project');
			$this->db->where(array('entity_id' => $entity_id, 'invoice_type !=' => 'N'));
			$this->db->where_in('project_id', $data['project_ids']);
			$query = $this->db->get();
			$pro_result = $query->result_array();

			$projects 	= array();
			foreach ($pro_result as $pro) {
				$projects[$pro['invoice_type']][] = $pro['project_id'];
			}
			
			// round hours based on entity setting
			/*$round_to = $this->ot_app->get('time_round_to');
			switch ($round_to){
				case 'F':
					$select= 'ROUND(sum(ts.hours)) AS quantity';
				break;
				case 'H':
					$select= 'ROUND( ROUND( sum(ts.hours)* 2 ) / 2,2) AS quantity';
				break;
				default:
					$select= 'sum(ts.hours) AS quantity';
			}*/
			$select= 'sum(ts.hours) AS quantity';
			$group_by = array();
			$description = array();
			$tmp_array = $data['group_by'];
			if ($tmp_array)
			{
				$group_field = array('project' => 'pr.project_id', 'tasktype' => 'ty.task_type_id', 'date' => '', 'activity' => 'ts.activity_name', 'member' => 'eu.user_id');
				$desc_field = array('project' => 'pr.project_name', 'tasktype' => 'ty.name', 'date' =>'"#"', 'activity' => 'ts.activity_name', 'member' => '" - ",eu.first_name," ", eu.last_name',);
				if (empty($data['start_date']) && empty($data['finish_date']) || $tmp_array['activity']){
					$desc_field['date'] = 	'case when DATEDIFF(min(DATE_FORMAT(ts.entry_date, "%Y-%m-%d")), max(DATE_FORMAT(ts.entry_date, "%Y-%m-%d"))) = 0 then concat(" [", DATE_FORMAT(min(ts.entry_date), "%d/%m/%Y"), "] ") else concat( " [", DATE_FORMAT(min(ts.entry_date), "%d/%m/%Y"), " to ", DATE_FORMAT(max(ts.entry_date), "%d/%m/%Y"), "] ") end';
				}
				foreach($group_field as $key => $val)
				{
					if ($tmp_array[$key]) {
						if ($val)
							$group_by[]	= $val;
						$description[]	= $desc_field[$key];
					}
				}
			}
			$description = ($description)?implode(",' ',",$description):'pr.project_code, pr.project_name';
			$group_by = ($group_by)?implode(',',$group_by):'';
			$sql = array();
			foreach ($projects as $inv_type=>$pro_ids)
			{
				$this->db->select($select, false);
				$this->db->select('concat('.$description.') AS description ', false);				
				$this->db->from('time_sheet AS ts');
				$this->db->join('project AS pr','ts.entity_id=pr.entity_id and pr.project_id = ts.project_id');
				$this->db->where(array('ts.entity_id' => $entity_id, 'ts.approval_status' => 'A', 'ts.is_billable' => 1, 'ts.is_progress !=' =>1));
				$this->db->where('ts.hours !=' , 0);
				if (!empty($data['start_date']) && !empty($data['finish_date']))
				{
					$this->db->where(array('ts.entry_date >=' => $data['start_date'], 'ts.entry_date <=' => $data['finish_date']));
				}
				$this->db->where_in('ts.project_id', $pro_ids);
				// project invoice type based rate selection
				switch ($inv_type)
				{
					case 'T':
							$this->db->select('pty.hourly_rate AS rate', false);
						break;
					case 'P':
							$this->db->select('pru.hourly_rate AS rate', false);
						break;
					case 'A':
							$this->db->select('pr.hourly_rate AS rate', false);
						break;
				}
				
				$this->db->join('task_type AS ty', 'ts.entity_id=ty.entity_id and ty.task_type_id=ts.task_type_id');
				$this->db->join('project_task_types AS pty', 'ts.entity_id=pty.entity_id and pty.project_id = ts.project_id AND pty.task_type_id=ts.task_type_id');
				$this->db->join('project_users as pru', 'pru.entity_id=pr.entity_id and ts.user_id=pru.user_id AND pru.project_id = pr.project_id');
				$this->db->join('entity_users AS eu', 'ts.entity_id=eu.entity_id and eu.user_id = ts.user_id');
				
				
				if (!empty($data['invoice_id'])) {	
					$this->db->where('(ts.invoice_id<=0 OR ts.invoice_id ='.$data['invoice_id'].')');
				} else {
					$this->db->where(array('ts.invoice_id <= '=>0));
				}

				if ($group_by)
					$this->db->group_by($group_by);
				// else
				$this->db->group_by('rate');
				$query	= $this->db->get();
				$sql[] = $this->db->last_query();
			}
			
			if ($group_by)
				$sql = 'SELECT re.* from ('.implode(' UNION ',$sql).') AS re group by re.description,re.rate';
			else
				$sql = 'SELECT re.quantity,re.rate,"Import description" as description  from ('.implode(' UNION ',$sql).') AS re group by re.rate';
				//$sql = 'SELECT CASE WHEN re.rate = 0 THEN sum(re.quantity) ELSE Round(sum(re.quantity * re.rate),2) END AS quantity, "Import description" as description , CASE WHEN re.rate = 0 THEN re.rate ELSE 1 END AS group_by, CASE WHEN re.rate = 0 THEN re.rate ELSE 1 END AS rate from ('.implode(' UNION ',$sql).') AS re group by group_by';			

			$query	= $this->db->query($sql);
			// echo $sql;exit;
			$result = $query->result_array();
		}
		
		// expenses detail by
		if (!empty($data['invoice_expenses'])) {
			$this->db->select('sum(pre.quantity) AS quantity');
			$this->db->from('project_expense AS pre');
			$this->db->join('project AS pr','pr.project_id = pre.project_id');
			$this->db->where_in('pre.project_id', $data['project_ids']);
			$this->db->where(array('pre.entity_id' => $entity_id, 'pre.approved_by > ' => '0'));
			
			switch ($data['detail_expenses'])
			{
				case 'C':
					$this->db->select('concat(pr.project_name," expenses : ", ety.name) AS description, if (pre.unit_price>0, pre.unit_price, pre.amount) AS rate', false);
					$this->db->join('expense_type AS ety', 'ety.expense_type_id=pre.expense_type_id');
					$this->db->group_by('pr.project_id, pre.expense_type_id, pre.unit_price');
				break;
				case 'U':
					$this->db->select('concat(pr.project_name," expenses : ", eu.first_name, " ", eu.last_name) AS description, if (pre.unit_price>0, pre.unit_price, pre.amount) AS rate', false);
					$this->db->join('entity_users AS eu', 'eu.user_id = pre.created_by');
					$this->db->group_by('pr.project_id, pre.created_by');
				break;
			}
			if (!empty($data['invoice_id'])) {	
				$this->db->where('(pre.invoice_id<=0 OR pre.invoice_id ='.$data['invoice_id'].')');
			} else {
				$this->db->where(array('pre.invoice_id <= '=>0));
			}
			$query	= $this->db->get();
			$expenses = $query->result_array();
			
			$result =array_merge($result, $expenses);
		}
		
		return $result;
	}
	
	public function get_client_details($data)
	{	
		$this->db->select('inv.client_contact_id, inv.currency_code, inv.address, inv.date_filter_type, inv.from_date,inv.to_date, inv.import_config, c.address as client_address');
		$this->db->from('client AS c');
		$this->db->join('invoice AS inv', 'inv.client_id = c.client_id', 'left');
		$this->db->where(array('c.entity_id' => $this->ot_app->get('entity_id'), 'c.client_id' => $data['client_id']));
		$this->db->order_by('inv.invoice_id', 'desc');
		$this->db->limit(1);
		
		$query	= $this->db->get();
		return $query->row_array();
	}

	public function filter_where($invoice_id)
	{
			$where['invoice_id']	= $invoice_id;
			$where['entity_id']		= $this->_entity_id;
			$where['created_by'] 	= $this->ot_user->get('user_id');
			$where['status']		= 'd';
			return $where;
	}

	public function delete($invoice_id)
	{
		$invoice_id = (int)$invoice_id;		
		if (!$invoice_id)
		{
			return false;
		}
		
		$inv_result	= $this->get_list("inv.invoice_number, inv.status", array('filters' => array('inv.invoice_id'=>$invoice_id)));

		$where = $this->filter_where($invoice_id);		
		if ($this->db->delete('invoice', $where)) {
			$this->db->delete('invoice_items', array('invoice_id' => $invoice_id));
			$this->db->delete('invoice_payment', array('invoice_id' => $invoice_id));
			
			// get project ids 
			$this->db->where(array('invoice_id' => $invoice_id, 'entity_id' => $this->_entity_id));
			$pro_ids = $this->db->select('DISTINCT project_id', false)->from('time_sheet')->get()->result_array();
			
			// reset invoiced timesheet hours
			if ($inv_result[0]['status']=='d') {
				$this->db->where(array('invoice_id' => $invoice_id, 'entity_id' => $this->_entity_id));
				$this->db->update('time_sheet', array('invoice_id' => 0, 'last_updated_date' => $this->auth->get_timezone_date('','UTC')));				
			}
			
			if ($pro_ids) {
				$tmp_ids = array();
				foreach ($pro_ids as $row) {
					$tmp_ids[] = $row['project_id'];
				}
				$this->reset_unin_hours($tmp_ids);
			}
			$result = $inv_result[0];
			$fullname = $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name');
			$act_data = array(
					'invoice_id' 	=> $invoice_id,
					'a_mode'		=> 'ID',
					'a_notes'		=>  $fullname. ' deleted Invoice #'.$result['invoice_number']
			);
			if (! invoice_activity($act_data))
			{
					return false;
			}
			return true;
		}
		// Insert invoice activity
		//$CI =& get_instance();

		return false;
	}
	
	public function get_last_invoices($client_id)
	{
		$this->db->select('max(t.invoice_id) as invoice_id, t.project_id');
		$this->db->from('time_sheet AS t');
		$this->db->join('project AS p','t.entity_id=p.entity_id and t.project_id=p.project_id','inner');
		$this->db->where(array('t.entity_id' => 'p.entity_id', 'p.client_id' => $client_id));
		$query = $this->db->get();
		
		//$result = array_key_index($query->result_array(), 'invoice_id');
		return $query->result_array();
	}
	
	public function archive_status($invoice_id, $status)
	{
		$this->db->where(array('invoice_id' => $invoice_id, 'created_by' => $this->ot_user->get('user_id')));
		$this->db->update('invoice', array('is_archive' => ($status)?1:0));

		return $this->db->affected_rows();
	}
	
	public function duplicate($invoice_id)
	{
		// get invoice data
		$this->db->select('*');
		$this->db->from('invoice');

		$where = $this->filter_where($invoice_id);

		$this->db->where($where);
		$query	= $this->db->get();
		$result =  $query->row_array();
		
		if ($result) {
			$unset_array = array('invoice_id','invoice_number','last_updated_by', 'last_updated_date', 'payment_amount');
			foreach ($unset_array as $key) unset($result[$key]);
			
			$result['invoice_number'] = $this->get_invoice_defaults();
//			$result['invoice_date'] = $this->auth->get_timezone_date();
			// get invoice items
			$details = $this->get_invoice_details($invoice_id);
			if ($details)
			{
				foreach ($details as $key=>$row)
				{
					unset($row['invoice_id']);
					$row['invoice_item_id'] = 0;
					$row['serial_no'] = 0;
					$result['details'][$key] = $row;
				}
			}
		}
		$result['status'] = 'd';
		$return = array();
		if ($return = $this->save($result))
		{
			$return['invoice_number']	= $result['invoice_number'];
		}
		
		return $return;
	}
}
