<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Client_model extends CI_Model {

	public $error	= '';

	private $_entity_id	= 0;

	function __construct()
	{
		parent::__construct();
		$this->load->library(OTBASE_PATH.'OT_App');
        $this->load->library(OTBASE_PATH.'OT_User');

        $this->_entity_id	= $this->ot_app->get('entity_id');
	}
	
	// return client list query
	public function clients_list_query($select, $options = array(), $all=false)
	{
		$this->db->select($select, false);
		$this->db->from('client AS c');

		$where	= array('c.entity_id' => $this->_entity_id);
		$this->db->join('project AS pr', 'pr.client_id = c.client_id', 'left');
		if (!is_admin() && !$all)
		{
// 			$this->db->join('project AS pr', 'pr.client_id = c.client_id');
			$this->db->join('project_users AS pru', 'pru.project_id = pr.project_id');
			$where['pru.user_id'] = $this->ot_user->get('user_id');
		}
		if (!empty($options['filters']))
		{
			foreach($options['filters'] as $key=>$val)
			{

				if (is_array($val))
					$this->db->where_in($key,$val);
				else
					$where[$key] = $val;
			}
		}

		$this->db->where($where);
		
		$order_by 			= !empty($options['order_by']) ? $options['order_by'] : 'c.client_name';
		$order_by_direction = !empty($options['order_by_direction']) ? $options['order_by_direction'] : 'asc';

		$this->db->order_by($order_by, $order_by_direction);

		$this->db->group_by('c.client_id');

		$from = !empty($options['from'])?$options['from']:0;
		if (!empty($options['count']))
		{
			$this->db->limit($options['count'], $from);
		}

		return $this->db->get();
	}

	// return clients in as array of standard array indexes
	public function get_clients($select='', $options=array(), $all=false)
	{
// 		$select = ($select)?$select:'c.client_id, concat(c.client_code, c.client_name) AS client_name, c.is_active';
		$select = ($select)?$select:'c.client_id, concat(c.client_code, c.client_name) AS client_name, c.is_active, sum(pr.budget_value) AS budget_value, sum(pr.actual_value) AS actual_value, c.bg_color';
		$query = $this->clients_list_query($select, $options, $all);
		
		return $query->result_array();
	}
	
	// return single client array based on client id 
	public function get_client($client_id, $select='client_id, client_name, client_code')
	{	
		$client_id = (int)$client_id;
		
		if (!$client_id)
		{
			$this->response->output_data('INVALID_REQUEST');
			return false;
		}
		
		$this->db->select($select);
		$this->db->from('client');
		$where	= array('entity_id' => $this->_entity_id, 'client_id' => $client_id);
		$this->db->where($where);
		$query = $this->db->get();
		
		return $query->row_array();
	}

	public function client_contacts_count($client_ids, $options = array())
    {
    	$this->db->select('client_id, count(*) as total_contacts');
		$this->db->from('client_contact');
    	$this->db->where(array('entity_id' => $this->_entity_id));
		$this->db->where_in('client_id',$client_ids);
		if (!empty($options['filters']))
		{
			$this->db->where($options['filters']);
		}
		$this->db->group_by('client_id');
		
		$query = $this->db->get();
		
		return $query->result_array();
    }
	
	
	public function save($data)
	{
		$this->load->library('client_table');
		
		$result = $this->client_table->save($data); 
		if (!$result)
		{
			return false;
		}
		
		return array('client_id' => $result);
	}
	
	// return client contacts array
	public function get_contacts($client_id, $select = '')
	{
		$client_id	= (int)$client_id;		
		if (!$client_id)
		{
			$this->response->output_data('INVALID_REQUEST');
			return false;
		}
		
		$select = (!$select)?'cc.client_contact_id, cc.client_id, cc.name, cc.title, a.address_id, a.email, a.workphone, a.mobile':$select;

		$this->db->select($select);
		$this->db->from('client AS c');
		$this->db->join('client_contact AS cc', 'c.entity_id=cc.entity_id AND c.client_id = cc.client_id');
		$this->db->join('address AS a', 'cc.entity_id=a.entity_id AND  cc.address_id = a.address_id');
		$this->db->where(array('c.entity_id' => $this->_entity_id, 'c.client_id' => $client_id));
		$query = $this->db->get();
		
		return $query->result_array();
	}
	
	// return invoice client contact details
	public function get_invoice_contacts($client_contact_id=null, $select=null)
	{
// 		$client_contact_id	= (int)$client_contact_id;		
		if (!$client_contact_id)
		{
			$this->response->output_data('INVALID_REQUEST');
			return false;
		}
		
		$select	= (!$select)?'cc.client_contact_id, cc.client_id, cc.name, cc.title, a.address_id, a.email, a.workphone, a.mobile':$select;

		$this->db->select($select);
		$this->db->from('client AS c');
		$this->db->join('client_contact AS cc', 'c.client_id = cc.client_id');
		$this->db->join('address AS a', 'cc.address_id = a.address_id');

		$this->db->where('c.entity_id', $this->_entity_id);
		if (is_array($client_contact_id))
		{
			$this->db->where_in('cc.client_contact_id', $client_contact_id);
		}
		else
		{
			$this->db->where('cc.client_contact_id', $client_contact_id);
		}

		$query = $this->db->get();
		
		return $query->result_array();
	}
	
	public function delete($client_id)
	{
		$client_id = (int)$client_id;
		
		if (!$client_id)
		{
			$this->error	= 'INVALID_REQUEST';
			return false;
		}

		if ($this->_get_projects_count($client_id))
		{
			$this->error	= 'CLIENT_HAVE_ACTIVE_OR_ARCHIVE_PROJECTS';
			return false;
		}

		$this->load->library('client_table');

		if (!$this->client_table->delete($client_id))
		{
			$this->error	= 'DELETE_FAILED';
			return false;
		}

		return true;
	}

	private function _get_projects_count($client_id=0)
	{
		$this->db->from('project');
		$this->db->where('entity_id', $this->_entity_id);
		$this->db->where('client_id', $client_id);
// 		$this->db->where('is_active', "1");

		return $this->db->count_all_results();
	}

	public function get_client_count($active = 0)
	{
		$this->db->select('count(client_id) as archive_count');
		$this->db->from('client');
		$this->db->where(array('entity_id' => $this->_entity_id, 'is_active'=>$active));
		$query = $this->db->get();

		$result = $query->row_array();
		return $result['archive_count'];
	}
	
	public function archive($client_id=0, $data=array())
	{
		if (empty($client_id))
		{
			$this->error	= 'CLIENT_ID_REQUIRED';
			return false;
		}

		if (empty($data))
		{
			$data['is_active']	= 0;
		}

		if ($data['is_active'] && !license_limit($client_id, 'clients')) {
			$this->response->error(sprintf(lang('UPGRADE_YOUR_VERSION'),'clients'));
			return false;
		}

		if ($this->_get_projects_count($client_id))
		{
			$this->error	= $data['is_active'] ? 'UNABLE_TO_UNARCHIVE_CLIENT_HAVE_PROJECTS' : 'UNABLE_TO_ARCHIVE_CLIENT_HAVE_PROJECTS';
			return false;
		}

		$this->db->where(array(
			'entity_id'	=> $this->_entity_id,
			'client_id'	=> $client_id
		));

		$this->db->update('client', array(
			'is_active'	=> $data['is_active']
		));

		return true;
	}

}

/* End of file mod.client.php */
